'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.getEnablePosition = exports.getWinHeight = exports.getWinWidth = exports.createPortal = exports.hasVerticalScroll = exports.hasHorizontalScroll = exports.requestAnimationFrame = exports.ANIMATION_INTERVAL = exports.ANIMATE_STATUS_HIDING = exports.ANIMATE_STATUS_SHOWN = exports.ANIMATE_STATUS_SHOWING = exports.ANIMATE_STATUS_INIT = exports.ANIMATE_STATUS_NONE = undefined;
exports.getHolder = getHolder;
exports.getScrollbarWidth = getScrollbarWidth;
exports.enableWinScrollBar = enableWinScrollBar;
exports.disableWinScrollBar = disableWinScrollBar;
exports.isInRect = isInRect;

var _reactDom = require('react-dom');

var _envUtil = require('./envUtil');

var ANIMATE_STATUS_NONE = exports.ANIMATE_STATUS_NONE = 0;
var ANIMATE_STATUS_INIT = exports.ANIMATE_STATUS_INIT = 1;
var ANIMATE_STATUS_SHOWING = exports.ANIMATE_STATUS_SHOWING = 2;
var ANIMATE_STATUS_SHOWN = exports.ANIMATE_STATUS_SHOWN = 3;
var ANIMATE_STATUS_HIDING = exports.ANIMATE_STATUS_HIDING = 4;

var BAMBOO_PORTALS_HOLDER = 'bmbo_portals_holder';
var $bambooHolder = void 0;

// =====================================================================================
// =                                   Window Holder                                   =
// =====================================================================================
if (_envUtil.canUseDOM) {
	// Clean up menu holder for hot module reload
	window.document.querySelectorAll('#' + BAMBOO_PORTALS_HOLDER).forEach(function (ele) {
		ele.remove();
	});

	// Set up menu holder
	$bambooHolder = window.document.createElement('div');
	$bambooHolder.id = BAMBOO_PORTALS_HOLDER;
	$bambooHolder.className = 'bmbo-portals-holder';
	// $bambooHolder.dataset.timestamp = Date.now();
	window.document.body.appendChild($bambooHolder);
}
function getHolder() {
	return $bambooHolder;
}

// =====================================================================================
// =                                     Animation                                     =
// =====================================================================================
var ANIMATION_INTERVAL = exports.ANIMATION_INTERVAL = 1000 / 60;

var delayByFrame = function delayByFrame(func) {
	if (window.requestAnimationFrame) {
		window.requestAnimationFrame(func);
	} else {
		setTimeout(func, ANIMATION_INTERVAL);
	}
};

var requestAnimationFrame = exports.requestAnimationFrame = function requestAnimationFrame(func) {
	var delayFrame = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;

	var count = delayFrame;

	function doAction() {
		if (count <= 0) {
			func();
			return;
		}

		count -= 1;
		delayByFrame(doAction);
	}

	doAction();
};

// =====================================================================================
// =                                      Scroll                                       =
// =====================================================================================
/**
 * Get Scroll bar size
 * @type {number}
 */
var scrollbarWidth = 0;
function getScrollbarWidth() {
	if (scrollbarWidth || !_envUtil.canUseDOM) return scrollbarWidth;

	var $body = window.document.body;
	var scrollDiv = document.createElement('div');
	scrollDiv.className = 'bmbo-scrollbar-measure';
	$body.appendChild(scrollDiv);
	scrollbarWidth = scrollDiv.offsetWidth - scrollDiv.clientWidth;
	scrollDiv.remove();
	return scrollbarWidth;
}

var hasHorizontalScroll = exports.hasHorizontalScroll = function hasHorizontalScroll() {
	var winWidth = window.innerWidth;
	var bodyWidth = window.document.body.scrollWidth;
	return winWidth < bodyWidth;
};

var hasVerticalScroll = exports.hasVerticalScroll = function hasVerticalScroll() {
	var winHeight = window.innerHeight;
	var bodyHeight = window.document.body.scrollHeight;
	return winHeight < bodyHeight;
};

var winScrollLocker = [];

function enableWinScrollBar(name) {
	if (!_envUtil.canUseDOM) return;

	if (name) {
		winScrollLocker = winScrollLocker.filter(function (n) {
			return n !== name;
		});
		if (winScrollLocker.length) return;
	} else {
		winScrollLocker = [];
	}

	document.body.style.paddingRight = '';
	document.body.style.overflowY = '';
}

function disableWinScrollBar(name) {
	if (!_envUtil.canUseDOM) return;
	if (!hasVerticalScroll()) return;

	if (name) {
		winScrollLocker.push(name);
	}

	document.body.style.paddingRight = getScrollbarWidth() + 'px';
	document.body.style.overflowY = 'hidden';
}

// =====================================================================================
// =                                     Position                                      =
// =====================================================================================
var createPortal = exports.createPortal = function createPortal(node) {
	if (!_envUtil.canUseDOM) return null;

	return (0, _reactDom.createPortal)(node, $bambooHolder);
};

var getWinWidth = exports.getWinWidth = function getWinWidth() {
	var winWidth = window.innerWidth;
	if (hasVerticalScroll()) winWidth -= getScrollbarWidth();
	return winWidth;
};

var getWinHeight = exports.getWinHeight = function getWinHeight() {
	var winHeight = window.innerHeight;
	if (hasHorizontalScroll()) winHeight -= getScrollbarWidth();
	return winHeight;
};

/**
 * Get enabled position for element.
 * @param surroundRect
 * @param targetRect
 * @param position
 * @returns {{x: number, y: number}}
 */
var getEnablePosition = exports.getEnablePosition = function getEnablePosition(surroundRect, targetRect) {
	var position = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'dr';

	if (typeof window === 'undefined') return { x: 0, y: 0 };

	var _surroundRect$width = surroundRect.width,
	    sw = _surroundRect$width === undefined ? 0 : _surroundRect$width,
	    _surroundRect$height = surroundRect.height,
	    sh = _surroundRect$height === undefined ? 0 : _surroundRect$height;

	var sx = surroundRect.left || 0;
	var sy = surroundRect.top || 0;
	var _targetRect$width = targetRect.width,
	    tw = _targetRect$width === undefined ? 0 : _targetRect$width,
	    _targetRect$height = targetRect.height,
	    th = _targetRect$height === undefined ? 0 : _targetRect$height;


	var winWidth = getWinWidth();
	var winHeight = getWinHeight();
	var scrollX = window.scrollX;
	var scrollY = window.scrollY;

	var isR = position.includes('r');
	var isB = position.includes('b');
	var isT = position.includes('t');
	var isL = position.includes('l');
	var isLorR = isL && !isR || !isL && isR;
	var isTorB = isT && !isB || !isT && isB;

	var target = {
		x: sx + scrollX,
		y: sy + scrollY,
		direct: ''
	};

	if (isL && !isR) target.x = sx - tw + scrollX;
	if (isR && !isL) target.x = sx + sw + scrollX;
	if (isT && !isB) target.y = sy - th + scrollY;
	if (isB && !isT) target.y = sy + sh + scrollY;

	if (isL && isR) target.x = sx - (tw - sw) / 2 + scrollX;
	if (isT && isB) target.y = sy - (th - sh) / 2 + scrollY;

	target._x = target.x;
	target._y = target.y;

	// Bottom out of the window
	if (target.y - scrollY + th > winHeight) {
		if (isLorR) {
			target.y = winHeight - th + scrollY;
		} else {
			var ty = sy - th;
			if (ty >= 0) {
				target.y = sy + scrollY - th;
				target.direct = 'up';
			}
		}
	}

	// Top out of the window
	if (target.y - scrollY < 0) {
		if (isLorR) {
			target.y = scrollY;
		} else {
			target.y = sy + sh + scrollY;
		}
	}

	// Left out of the window
	if (target.x - scrollX < 0) {
		if (isTorB) {
			target.x = scrollX;
		} else {
			target.x = sx + sw + scrollX;
		}
	}

	// Right out of the window
	if (target.x - scrollX + tw > winWidth) {
		if (isTorB) {
			target.x = winWidth - tw + scrollX;
		} else {
			target.x = sx + scrollX - tw;
		}
	}

	return target;
};

/**
 * Get enabled position for element.
 * @param surroundRect
 * @param targetRect
 * @param position
 * @param switchPos		Switch position when position has no space left
 * @returns {{x: number, y: number}}
 */
/* export const getEnablePosition2 = (surroundRect, targetRect, position = 'dr', switchPos = true) => {
	if (typeof window === 'undefined') return { x: 0, y: 0 };

	const { width: sw = 0, height: sh = 0 } = surroundRect;
	const sx = surroundRect.left || 0;
	const sy = surroundRect.top || 0;
	const { width: tw = 0, height: th = 0 } = targetRect;

	const winWidth = getWinWidth();
	const winHeight = getWinHeight();
	const scrollX = window.scrollX;
	const scrollY = window.scrollY;

	const isR = position.includes('r');
	const isB = position.includes('b');
	const isT = position.includes('t');
	const isL = position.includes('l');

	const target = {
		x: sx + scrollX,
		y: sy + scrollY,
		direct: '',
	};

	if (isL && !isR) target.x = (sx - tw) + scrollX;
	if (isR && !isL) target.x = (sx + sw) + scrollX;
	if (isT && !isB) target.y = (sy - th) + scrollY;
	if (isB && !isT) target.y = (sy + sh) + scrollY;

	if (isL && isR) target.x = (sx - ((tw - sw) / 2)) + scrollX;
	if (isT && isB) target.y = (sy - ((th - sh) / 2)) + scrollY;

	target._x = target.x;
	target._y = target.y;

	// Bottom out of the window
	if ((target.y - scrollY) + th > winHeight) {
		if (isR) {
			target.y = (winHeight - th) + scrollY;
		} else {
			const ty = (sy + scrollY) - th;
			if (ty >= 0) {
				target.y = (sy + scrollY) - th;
				target.direct = 'up';
			}
		}
	}
	// Top out of the window
	if ((target.y - scrollY) < 0) {
		if (switchPos) {
			target.y = sy + sh + scrollY;
		} else {
			target.y = scrollY;
		}
	}

	// Left out of the window
	if (target.x - scrollX < 0) {
		if (switchPos) {
			target.x = sx + sw;
		} else {
			target.x = scrollX;
		}
	}

	// Right out of the window
	if ((target.x - scrollX) + tw > winWidth) {
		if (switchPos) {
			target.x = sx - tw;
		} else {
			target.x = (winWidth + scrollX) - tw;
		}
	}

	return target;
}; */

function isInRect(x, y, rect) {
	return rect.left < x && x < rect.right && rect.top < y && y < rect.bottom;
}